from .core import UNDEFINED, NODATA_TIME, NODATA_NEGATIVE

NODATA_FLOAT = UNDEFINED
NODATA_INT = NODATA_NEGATIVE
PRECIP_2_BYTE = 101001
UNDEFINED_COMPRESSION = 0
NO_COMPRESSION = 1
ZLIB_DEFLATE_COMPRESSION = 26
UNDEFINED_PROJECTION_DATUM = 0
NAD_27 = 1
NAD_83 = 2
DSS7_GRID_STRUCT_VERSION = 100
GRID_STRUCT_VERSION_UNDEFINED = -1
