
      integer MESS_NONE
      integer MESS_CRITICAL
      integer MESS_TERSE
      integer MESS_GENERAL
      integer MESS_USER_DIAG
      integer MESS_INTERNAL_DIAG_1
      integer MESS_INTERNAL_DIAG_2

      data MESS_NONE			/0/
      data MESS_CRITICAL		/1/
      data MESS_TERSE			/2/
      data MESS_GENERAL			/3/
      data MESS_USER_DIAG		/4/
      data MESS_INTERNAL_DIAG_1	/5/
      data MESS_INTERNAL_DIAG_2	/6/



      integer zmessaging_general_ID
      integer zmessaging_global_ID
      integer zmessaging_get_ID
      integer zmessaging_put_ID
      integer zmessaging_read_ID
      integer zmessaging_write_ID
      integer zmessaging_perm_ID
      integer zmessaging_open_ID
      integer zmessaging_check_ID
      integer zmessaging_locking_ID
      integer zmessaging_tsread_ID
      integer zmessaging_tswrite_ID
      integer zmessaging_alias_ID
      integer zmessaging_copy_ID
      integer zmessaging_utility_ID
      integer zmessaging_catalog_ID
      integer zmessaging_filecheck_ID
      integer zmessaging_jni_ID
      integer zmessaging_undefined_ID

      data zmessaging_general_ID		/0/
      data zmessaging_global_ID			/1/
      data zmessaging_get_ID			/2/
      data zmessaging_put_ID			/3/
      data zmessaging_read_ID			/4/
      data zmessaging_write_ID			/5/
      data zmessaging_perm_ID			/6/
      data zmessaging_open_ID			/7/
      data zmessaging_check_ID			/8/
      data zmessaging_locking_ID		/9/
      data zmessaging_tsread_ID			/10/
      data zmessaging_tswrite_ID		/11/
      data zmessaging_alias_ID			/12/
      data zmessaging_copy_ID			/13/
      data zmessaging_utility_ID		/14/
      data zmessaging_catalog_ID		/15/
      data zmessaging_filecheck_ID		/16/
      data zmessaging_jni_ID			/17/
      data zmessaging_undefined_ID		/18/

