//  Rename me to something like function names
#ifndef ZDSS_RECORD_DESCRIPTIONS_H
#define ZDSS_RECORD_DESCRIPTIONS_H


#define RECORD_TYPE_100  "Regular-interval time series"
#define RECORD_TYPE_101  "Regular-interval time series pattern"
#define RECORD_TYPE_105  "Regular-interval time series doubles"
#define RECORD_TYPE_110  "Irregular-interval time series"
#define RECORD_TYPE_111  "Irregular-interval time series pattern"
#define RECORD_TYPE_115  "Irregular-interval time series doubles"
#define RECORD_TYPE_200  "Paired Data"
#define RECORD_TYPE_205  "Paired Data doubles"
#define RECORD_TYPE_300  "Text Data"
#define RECORD_TYPE_400  "Gridded - Undefined grid with time"
#define RECORD_TYPE_401  "Gridded - Undefined grid"
#define RECORD_TYPE_410  "Gridded - HRAP grid with time reference"
#define RECORD_TYPE_411  "Gridded - HRAP grid"
#define RECORD_TYPE_420  "Gridded - Albers with time reference"
#define RECORD_TYPE_421  "Gridded - Albers"
#define RECORD_TYPE_430  "Gridded - SHG with time reference"
#define RECORD_TYPE_431  "Gridded - SHG"
#define RECORD_TYPE_UNDEFINED  "Undefined data type"


#endif //  ZDSS_RECORD_DESCRIPTIONS_H

